@@ -49,7 +49,7 @@
 #include "defs.h"
 #include "netio.h"
 #include "pdns.h"
-#if WANT_PDNS_DNSDB
+#if WANT_PDNS_DNSDB || WANT_PDNS_DNSDB2
 #include "pdns_dnsdb.h"
 #endif
 #if WANT_PDNS_CIRCL
@@ -137,7 +137,7 @@ main(int argc, char *argv[]) {
 	/* process the command line options. */
 	while ((ch = getopt(argc, argv,
 			    "R:r:N:n:i:M:u:p:t:b:k:J:V:"
-			    "dfhIjmqSsUv8" QPARAM_GETOPT))
+			    "dfhIjmqSsUv468" QPARAM_GETOPT))
 	       != -1)
 	{
 		switch (ch) {
@@ -389,6 +389,12 @@ main(int argc, char *argv[]) {
 		case 'h':
 			help();
 			my_exit(0);
+		case '4':
+			curl_ipresolve = CURL_IPRESOLVE_V4;
+			break;
+		case '6':
+			curl_ipresolve = CURL_IPRESOLVE_V6;
+			break;
 		case '8':
 			allow_8bit = true;
 			break;
@@ -630,7 +636,7 @@ static void
 help(void) {
 	verb_ct v;
 
-	printf("usage: %s [-cdfgGhIjmqSsUv8] [-p dns|json|csv]\n",
+	printf("usage: %s [-cdfgGhIjmqSsUv468] [-p dns|json|csv]\n",
 	       program_name);
 	puts("\t[-k (first|last|duration|count|name|data)[,...]]\n"
 	     "\t[-l QUERY-LIMIT] [-L OUTPUT-LIMIT] [-A after] [-B before]\n"
@@ -673,10 +679,17 @@ help(void) {
 	     "\t-s/-S can be repeated before several -k arguments.\n"
 	     "use -U to turn off SSL certificate verification.\n"
 	     "use -v to show the program version.\n"
+	     "use -4 to force connecting to the server via IPv4.\n"
+	     "use -6 to force connecting to the server via IPv6.\n"
 	     "use -8 to allow arbitrary 8-bit values in -r and -n arguments");
 
 	puts("for -u, system must be one of:");
+#if WANT_PDNS_DNSDB
 	puts("\tdnsdb");
+#endif
+#if WANT_PDNS_DNSDB2
+	puts("\tdnsdb2");
+#endif
 #if WANT_PDNS_CIRCL
 	puts("\tcircl");
 #endif
@@ -697,6 +710,12 @@ pick_system(const char *name) {
 	if (strcmp(name, "dnsdb") == 0)
 		return pdns_dnsdb();
 #endif
+#if WANT_PDNS_DNSDB2
+	if (strcmp(name, "dnsdb2") == 0) {
+		encap = encap_saf;
+		return pdns_dnsdb2();
+	}
+#endif
 #if WANT_PDNS_CIRCL
 	if (strcmp(name, "circl") == 0)
 		return pdns_circl();
@@ -932,6 +951,10 @@ read_configs(void) {
 			     "echo dnsdb apikey $APIKEY;"
 			     "echo dnsdb server $DNSDB_SERVER;"
 #endif
+#if WANT_PDNS_DNSDB2
+			     "echo dnsdb2 apikey $APIKEY;"
+			     "echo dnsdb2 server $DNSDB_SERVER;"
+#endif
 #if WANT_PDNS_CIRCL
 			     "echo circl apikey $CIRCL_AUTH;"
 			     "echo circl server $CIRCL_SERVER;"
@@ -1046,6 +1069,10 @@ do_batch(FILE *f, qparam_ct qpp) {
 		if (nl != NULL)
 			*nl = '\0';
 
+		/* allow # as a comment syntax */
+		if (command[0] == '#')
+			continue;
+
 		DEBUG(1, true, "do_batch(%s)\n", command);
 
 		/* if this is a $OPTIONS, parse it and change our qparams. */
